from fastapi import APIRouter
from pydantic import BaseModel
import azure.cosmos.exceptions as exceptions
from ..database.mycosmosdb import callConnect
import logging


class Fund(BaseModel):
    oid: str
    city: str
    fund: str
    fundstartdate: str
class TMets(BaseModel):
    oid: str
    fund: str
class TMetDelete(BaseModel):
    oid: str
    fund: str
    metric: str
class TMetsEdit(BaseModel):
    oid: str
    fund: str
    metric: str
    m1: bool
    m2: bool
    m3: bool
    m4: bool
class Passport(BaseModel):
    oid: str
    city: str




def read_item(container, doc_id, account_number):
    response = container.read_item(item=doc_id, partition_key=account_number)
    return response



fundroute = APIRouter()



"""
Endpoint used to register a new fund.
We check if there are already funds present so we append to the list or we create the new dictionary to house the fund and any new funds added in the future.
"""
@fundroute.post('/newfund')
async def register_fund(fund: Fund):
    ndate = f"{fund.fundstartdate[5:7]}-{fund.fundstartdate[8:10]}-{fund.fundstartdate[:4]}"
    try:
        db_result = read_item(callConnect.container, fund.oid, fund.city)
        
    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /newfund container')
        return {"url": "/", "atype": False}


    target2 = {
        "id": fund.oid,
        "name": fund.fund,
        "targetmetrics": []
    }
    callConnect.container6.upsert_item(body=target2)

    if "funds" in db_result:
        user2 = {
            "id": db_result["id"],
            "city": db_result["city"],
            "email": db_result["email"],
            "session_id": db_result["session_id"],
            "sub_created": db_result["sub_created"],
            "plan_id": db_result["plan_id"],
            "funds": db_result["funds"],
            "subs": db_result["subs"]
        }
        nfunds = db_result["funds"]
        nfunds.append({"fund": fund.fund, "fundstartdate": ndate, "companies": [], "pending": []})
        fund = {
            "funds": nfunds
        }
    else:
        user2 = {
            "id": db_result["id"],
            "city": db_result["city"],
            "email": db_result["email"],
            "session_id": db_result["session_id"],
            "sub_created": db_result["sub_created"],
            "plan_id": db_result["plan_id"],
            "subs": db_result["subs"]
            }
        fund = {
        "funds": [
            {
                "fund": fund.fund,
                "fundstartdate": ndate,
                "companies": [],
                "pending": []
            }
        ]
    }

    user2.update(fund)
    callConnect.container.upsert_item(body=user2)
    return {"url": "/dashboard/fundoverview"}


"""
Endpoint used to update the target metrics.
"""
@fundroute.post('/gettargetmetrics')
async def get_target_metrics(tmet: TMets):
    try:
        db_result = read_item(callConnect.container6, tmet.oid, tmet.fund)
    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /gettargetmetrics container')
        return {"url": "/", "atype": False}
    

    if "targetmetrics" in db_result:
        return {"tmets": db_result["targetmetrics"]}
    

"""
Endpoint used to update the target metrics.
"""
@fundroute.post('/edittargetmetrics')
async def edit_target_metrics(tmet: TMetsEdit):
    try:
        db_result = read_item(callConnect.container6, tmet.oid, tmet.fund)
    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /edittargetmetrics container6')
        return {"url": "/", "atype": False}


    if "targetmetrics" in db_result:
        if tmet.metric == "Greenhouse Gas Emissions":
            user = {
            "id": db_result["id"],
            "name": db_result["name"],
            "targetmetrics": []
            }
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Greenhouse Gas Emissions"]
            newMet = {
                "label": "Greenhouse Gas Emissions",
                "metrics": [
                    {
                        "label": "Scope 1 Greenhouse Gas Emissions",
                        "value": tmet.m1
                    },
                    {
                        "label": "Scope 2 Greenhouse Gas Emissions",
                        "value": tmet.m2
                    },
                    {
                        "label": "Scope 3 Greenhouse Gas Emissions",
                        "value": tmet.m3
                    }
                ]
            }
            mets.append(newMet)
            target = {
                "targetmetrics": mets
            }
            user.update(target)
            callConnect.container6.upsert_item(body=user)
        elif tmet.metric == "Water Use":
            user = {
            "id": db_result["id"],
            "name": db_result["name"],
            "targetmetrics": []
            }
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Water Use"]
            newMet = {
                "label": "Water Use",
                "metrics": [
                    {
                        "label": "Water Consumption",
                        "value": tmet.m1
                    },
                    {
                        "label": "Water Withdrawn by Baseline Water Stress",
                        "value": tmet.m2
                    }
                ]
            }
            mets.append(newMet)
            target = {
                "targetmetrics": mets
            }
            user.update(target)
            callConnect.container6.upsert_item(body=user)
        elif tmet.metric == "Waste":
            user = {
            "id": db_result["id"],
            "name": db_result["name"],
            "targetmetrics": []
            }
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Waste"]
            newMet = {
                "label": "Waste",
                "metrics": [
                    {
                        "label": "Waste Creation",
                        "value": tmet.m1
                    },
                    {
                        "label": "Hazardous Waste",
                        "value": tmet.m2
                    },
                    {
                        "label": "Recycled Waste",
                        "value": tmet.m3
                    }
                ]
            }
            mets.append(newMet)
            target = {
                "targetmetrics": mets
            }
            user.update(target)
            callConnect.container6.upsert_item(body=user)
        elif tmet.metric == "Energy Use":
            user = {
            "id": db_result["id"],
            "name": db_result["name"],
            "targetmetrics": []
            }
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Energy Use"]
            newMet = {
                "label": "Energy Use",
                "metrics": [
                    {
                        "label": "Energy Consumption",
                        "value": tmet.m1
                    },
                    {
                        "label": "Renewable Energy Use",
                        "value": tmet.m2
                    },
                    {
                        "label": "Grid Electricity Use",
                        "value": tmet.m3
                    }
                ]
            }
            mets.append(newMet)
            target = {
                "targetmetrics": mets
            }
            user.update(target)
            callConnect.container6.upsert_item(body=user)
        elif tmet.metric == "Employee Diversity":
            user = {
            "id": db_result["id"],
            "name": db_result["name"],
            "targetmetrics": []
            }
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Employee Diversity"]
            newMet = {
                "label": "Employee Diversity",
                "metrics": [
                    {
                        "label": "Gender Representation",
                        "value": tmet.m1
                    },
                    {
                        "label": "Sexual Orientation Representation",
                        "value": tmet.m2
                    },
                    {
                        "label": "Race & Ethnicity Representation",
                        "value": tmet.m3
                    }
                ]
            }
            mets.append(newMet)
            target = {
                "targetmetrics": mets
            }
            user.update(target)
            callConnect.container6.upsert_item(body=user)
        elif tmet.metric == "Employee Turnover":
            user = {
            "id": db_result["id"],
            "name": db_result["name"],
            "targetmetrics": []
            }
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Employee Turnover"]
            newMet = {
                "label": "Employee Turnover",
                "metrics": [
                    {
                        "label": "Involuntary Turnover Rate",
                        "value": tmet.m1
                    },
                    {
                        "label": "Voluntary Turnover Rate",
                        "value": tmet.m2
                    },
                    {
                        "label": "New Employee Turnover Rate",
                        "value": tmet.m3
                    }
                ]
            }
            mets.append(newMet)
            target = {
                "targetmetrics": mets
            }
            user.update(target)
            callConnect.container6.upsert_item(body=user)
        elif tmet.metric == "Compensation & Job Creation":
            user = {
            "id": db_result["id"],
            "name": db_result["name"],
            "targetmetrics": []
            }
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Compensation & Job Creation"]
            newMet = {
                "label": "Compensation & Job Creation",
                "metrics": [
                    {
                        "label": "Median Worker Pay",
                        "value": tmet.m1
                    },
                    {
                        "label": "CEO-to-Median Worker Pay Ratio",
                        "value": tmet.m2
                    },
                    {
                        "label": "Job Creation",
                        "value": tmet.m3
                    }
                ]
            }
            mets.append(newMet)
            target = {
                "targetmetrics": mets
            }
            user.update(target)
            callConnect.container6.upsert_item(body=user)
        elif tmet.metric == "Board Diversity":
            user = {
            "id": db_result["id"],
            "name": db_result["name"],
            "targetmetrics": []
            }
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Board Diversity"]
            newMet = {
                "label": "Board Diversity",
                "metrics": [
                    {
                        "label": "Gender Representation",
                        "value": tmet.m1
                    },
                    {
                        "label": "Sexual Orientation Representation",
                        "value": tmet.m2
                    },
                    {
                        "label": "Race & Ethnicity Representation",
                        "value": tmet.m3
                    }
                ]
            }
            mets.append(newMet)
            target = {
                "targetmetrics": mets
            }
            user.update(target)
            callConnect.container6.upsert_item(body=user)
        elif tmet.metric == "Board Structure":
            user = {
            "id": db_result["id"],
            "name": db_result["name"],
            "targetmetrics": []
            }
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Board Structure"]
            newMet = {
                "label": "Board Structure",
                "metrics": [
                    {
                        "label": "Number of Boards Served By Directors",
                        "value": tmet.m1
                    },
                    {
                        "label": "Independence of the Compensation Committee",
                        "value": tmet.m2
                    },
                    {
                        "label": "Independence of the Board Chair",
                        "value": tmet.m3
                    },
                    {
                        "label": "Director Independence",
                        "value": tmet.m4
                    }
                ]
            }
            mets.append(newMet)
            target = {
                "targetmetrics": mets
            }
            user.update(target)
            callConnect.container6.upsert_item(body=user)
        else:
            pass

"""
Endpoint used to update the target metrics.
"""
@fundroute.post('/removetargetmetrics')
async def remove_target_metrics(tmet: TMetDelete):
    try:
        db_result = read_item(callConnect.container6, tmet.oid, tmet.fund)
    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /removetargetmetrics container6')
        return {"url": "/", "atype": False}
    
    if "targetmetrics" in db_result:
        if tmet.metric == "Greenhouse Gas Emissions":
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Greenhouse Gas Emissions"]
            user = {
                "id": db_result["id"],
                "name": db_result["name"],
                "targetmetrics": mets
            }
            callConnect.container6.upsert_item(body=user)
        elif tmet.metric == "Water Use":
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Water Use"]

            user = {
                "id": db_result["id"],
                "name": db_result["name"],
                "targetmetrics": mets
            }
            
            callConnect.container6.upsert_item(body=user)

        elif tmet.metric == "Waste":
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Waste"]

            user = {
                "id": db_result["id"],
                "name": db_result["name"],
                "targetmetrics": mets
            }
            
            callConnect.container6.upsert_item(body=user)

        elif tmet.metric == "Energy Use":
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Energy Use"]

            user = {
                "id": db_result["id"],
                "name": db_result["name"],
                "targetmetrics": mets
            }
            
            callConnect.container6.upsert_item(body=user)

        elif tmet.metric == "Employee Diversity":
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Employee Diversity"]
            
            user = {
                "id": db_result["id"],
                "name": db_result["name"],
                "targetmetrics": mets
            }
            
            callConnect.container6.upsert_item(body=user)

        elif tmet.metric == "Employee Turnover":
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Employee Turnover"]

            user = {
                "id": db_result["id"],
                "name": db_result["name"],
                "targetmetrics": mets
            }
            
            callConnect.container6.upsert_item(body=user)

        elif tmet.metric == "Compensation & Job Creation":
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Compensation & Job Creation"]

            user = {
                "id": db_result["id"],
                "name": db_result["name"],
                "targetmetrics": mets
            }
            
            callConnect.container6.upsert_item(body=user)

        elif tmet.metric == "Board Diversity":
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Board Diversity"]
            
            user = {
                "id": db_result["id"],
                "name": db_result["name"],
                "targetmetrics": mets
            }

            callConnect.container6.upsert_item(body=user)

        elif tmet.metric == "Board Structure":
            mets = [z for z in db_result["targetmetrics"] if z["label"] != "Board Structure"]

            user = {
                "id": db_result["id"],
                "name": db_result["name"],
                "targetmetrics": mets
            }
            
            callConnect.container6.upsert_item(body=user)



"""
Endpoint used to grab the funds that a user has
"""
@fundroute.post('/getfund')
def subscription_check(passport: Passport):
    try:
        db_result = read_item(callConnect.container, passport.oid, passport.city)
        
    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /getfund container')
        return {"url": "/", "atype": False}

    if "funds" in db_result:
        if "subs" in db_result:
            return {"funds": db_result["funds"], "subs": db_result["subs"]}
        else:
            return {"funds": db_result["funds"]}


